/************************************************************
  cdrbk.c ( CDROM disk deiver )
  Copyright (C) 2001 Nobuhiro Sakawa
  Date: 2001/06/14
  Revision: 1.0 $
*************************************************************/
/*
cdbrk.c 			CDROM Disk READ
*/

#include "ide.h"
#include "bios.h"
#include "config.h"
#define BLOCK_SIZE 2048
#define SYSERR		-1
#ifndef FALSE
#define FALSE		0
#endif
#ifndef TRUE
#define TRUE		1
#endif
#ifndef NULL
#define NULL		0
#endif

#ifdef 	READ
#undef	READ
#endif
#define READ 0

#ifdef	WRITE
#undef	WRITE
#endif
#define WRITE 1

/************************* debug mode *************************/
#if 0
#define	DBG 
#endif

extern int boot_dev;
extern int boot_partition;
extern int minor_device;
extern int disk_error;

struct geometry cd0_info;
extern int cd_bmul;
static int error;
static struct drive_info driveinfo;
static unsigned char datablock[512];
static unsigned int iobase;   /* we boot off the Secondary disk only */
static unsigned int command;  /* command port */
static unsigned char *IOBASE ;
extern unsigned int devtype;
#define IDE_DEV 0
#define CD_DEV  1
#define LBA_DEV 2

typedef struct {
	unsigned char packet[12];
	char *address;
	int  count;	
	int drive;
	int status;
	int error;
} PACKET_INFO;
	
static char sense_data[18];

static	void wait()
{
	int	i;
	for(i=0;i<2;i++) {}
}

static	void wait500ms()
{
	unsigned long	about500ms = ((500*100)*10);
	unsigned int	i ;
	for ( i = 0 ; i<about500ms ; i++) {}
}

#if 0
static void	outb(unsigned short iobase, unsigned char val) 
{
	wait(); wait();
	*( IOBASE + iobase ) = (unsigned char)val ;
}

static unsigned char inb(unsigned short iobase) 
{
	unsigned char	rc ;
	wait(); wait();
	rc = (unsigned char)*( IOBASE + iobase );
	return ( rc ) ;
}

static insw(short port, short *buffer, int count)
{
	int	i;
	unsigned short	*rp ;
	rp = (unsigned short*)(IOBASE + port) ;
	for(i=0;i<count;i++)
	{
		wait();
		*buffer++ = *rp;
	}
}

static outsw(short port, short *buffer, int count)
{
	int	i;
	unsigned short	*rp ;
	rp = (unsigned short*)(IOBASE + port) ;
	for(i=0;i<count;i++)
	{
		wait();
		*rp = *buffer++;
	}
}
#endif

static void bzero(char *dest, int c)
{
    while(c--) {
      *((char *)dest)++ = 0;
    }
}

static void bcopy(char *dest,char *src,int c)
{
    while(c--) {
      *((char *)dest)++ = *((char *)src)++;
    }
}

static void cdwait()
{
	while (inb(iobase + STATUS_REG) & BSY_STATUS)
		;
}

static	void wait10ms()
{
	unsigned long	about10ms = ((10*100)*10);
	unsigned int	i ;
	for ( i = 0 ; i<about10ms ; i++) {}
}

static int time_cdwait()
{
	unsigned int i;
	for(i=0; i<5000; i++) {
		if (!(inb(iobase + STATUS_REG) & BSY_STATUS)) {
			return 0;
		}
		wait10ms();
	}
	return 1;
}

void atapi_command( PACKET_INFO *p)
{
	int i;
	PACKET_INFO sense_info;
	unsigned short byte_count;
	unsigned char *buf;
	int words;
	unsigned char ireason;
	unsigned char status;
	unsigned short count;

	if (time_cdwait()) { 
		sshprintk("ATAPI Dvice Error\n");
		p->status = 1; 
		return; 
	}
	outb(iobase + FT_REG, 0);
	outb(iobase + SECTOR_COUNT, 0);
	outb(iobase + SECTOR_NUMBER, 0);
	outb(iobase + HCYLINDER_NUMBER, (p->count >> 8) & 0xff);
	outb(iobase + LCYLINDER_NUMBER, p->count & 0xff);
	outb(iobase + DH_REG, 0xA0 | ((p->drive & 1) << 4));
	outb(iobase + COMMAND_REG, ATAPI_PACKET_COMMAND);
	while (!(inb(iobase + STATUS_REG) & DRQ_STATUS));
	outsw(iobase + DATA_REG, (short *)p->packet, 12/2);
	time_cdwait();
	status  = inb(iobase + STATUS_REG);
	if (status & 1) {
		p->status = status;
		p->error = inb(iobase + ERROR_REG);
		wait500ms(); wait500ms();
/*		sshprintk("ATAPI Command Error!!\n"); */
		return;
	}
	if (status & DRQ_STATUS) {	/* Data to transfer in from adaptor */
		buf = p->address;
		words = p->count/2;
		insw(iobase + DATA_REG, (short *)p->address, words);
		time_cdwait();
		status  = inb(iobase + STATUS_REG);
	}
	else {
/*		sshprintk("ATAPI Command Error!!\n"); */
	}
	p->status = inb(iobase + STATUS_REG);
}

void atapi_request_sense( int drive, PACKET_INFO *p, char *sens_data)
{
	bzero((char *)p, sizeof(PACKET_INFO));
	p->packet[0] = 0x03;  /* REQUEST_SENCE */
	p->packet[4] = 18;
	p->count = 18;
	p->drive = drive;
	p->address = sense_data;
	atapi_command(p);
}

void test_unit_ready(PACKET_INFO *p, int drive)
{
	bzero((char *)p, sizeof(PACKET_INFO));
	p->packet[0] = 0x00;  /*TEST_UNIT_READY*/
	p->drive = drive;
	atapi_command(p);
}

static void set_features(int drive)
{
	outb(iobase + FT_REG, 0x03);				/* set features sub command */
	outb(iobase + SECTOR_COUNT, 0x08 | 0x03);   /* pio mode = 0 */
	outb(iobase + SECTOR_NUMBER, 0x00);
	outb(iobase + LCYLINDER_NUMBER, 0x00);
	outb(iobase + HCYLINDER_NUMBER, 0x00);
	outb(iobase + DH_REG,
				0xa0 | (drive << 4) );
	outb(iobase + COMMAND_REG, 0xef );  		/* SET FEATURES */
	cdwait();
}

static int identify_device(int drive, struct drive_info *driveinfo)
{
	struct drive_information *infoptr; 

	outb(iobase + DH_REG, 0xa0 | drive<<4); 
	outb(iobase + COMMAND_REG, ATAPI_IDENTIFY_DEVICE);  /* get drive information CDROM */
	cdwait();
	insw(iobase + DATA_REG, (short *)datablock, 256);
	infoptr = (struct drive_information *) datablock;
	driveinfo->devinfo.cylinders = infoptr->now_num_sectors; 
	driveinfo->devinfo.heads     = infoptr->now_num_heads; 
	driveinfo->devinfo.sectors   = infoptr->now_num_sectors_per_track; 
/*
	sshprintk(" Boot Device  type  name \n")
	hwprintk(infoptr->capabilities);
	sshprintk(infoptr->model_number);
	dbcommand(0,datablock,datablock + 256);
*/
	return 0;
}

int cd_open() 
{
	int drive, partition,i;
	unsigned char test_byte;
	unsigned char	block[BLOCK_SIZE];
	unsigned char	*unit_adress;
	unsigned long   wtmp;
	PACKET_INFO p;

	drive     = minor_device;
	partition = boot_partition;
	driveinfo.devinfo = * (struct geometry *)&cd0_info ;
	IOBASE  = (unsigned char*) IO_WIN;
	if (boot_dev == 1) {
		iobase 	= IO_BASE_OFFSET ;	/* I/O port */
	} else {
		iobase 	= IO_BASE_OFFSET_IDE ;	/* Super I/O port */
	}
	command = 0xe ; 		/* command port */
	
	outb(iobase + DH_REG, 0xa0 | drive<<4);
	if ((inb(iobase + HCYLINDER_NUMBER) == 0xeb) &&
		(inb(iobase + LCYLINDER_NUMBER) == 0x14) ) {
		sshprintk("Boot Device ATAPI !! \n");
		devtype = CD_DEV; 
	}
	else {
		sshprintk("Boot Device Not CDROM !!\n");
		return -1;
	}

	if (identify_device(drive,&driveinfo) == SYSERR) {
		sshprintk("identify device not supported on drive ");
		hwprintk(drive); sshprintk("\n");
	}
	/* ATAPI reset */
	outb(iobase + COMMAND_REG, ATAPI_SOFT_RESET);
	cdwait();
	/* PIO mode set */
	set_features(drive);

	test_unit_ready(&p, drive);	
	for (i = 0; i < 20; i++) {
		test_unit_ready(&p, drive);
		if (!(p.status & 1)){
			sshprintk("CDROM is Ready !!\n");
			break; /* CDROM  is Ready */
		}
		sshprintk("CDROM is Not Ready\n");
		time_cdwait();
		wait500ms(); wait500ms(); wait500ms(); wait500ms();
		wait500ms(); wait500ms(); wait500ms(); wait500ms();
	}
	
	error = 0;
	bzero((char *) &p, sizeof(PACKET_INFO));
	p.packet[0] = 0x25;  /*READ_CAPACITY*/
	p.count = 8;
	p.drive = drive;
	p.address = block;
	atapi_command(&p);
	
	driveinfo.part[0].raw_size = (block[0] << 24) | (block[1] << 16) |
								 (block[2] << 8)  | block[3];
	driveinfo.part[0].start = 0;
	driveinfo.part[0].size = driveinfo.part[0].raw_size;

#ifdef DBG
sshprintk("cd_open() OK \n"0;
#endif
	return OK;
}

cd_close()
{
}

int cd_read_block(int block, char *address)
{
#ifdef DBG
sshprintk("cd_r_block="); hwprintk(block); sshprink("\n");
#endif
	return cd_r_block(1, block + driveinfo.part[0].start, 
						 minor_device, address);
}

int cd_r_block(int n, long block_num, int device, char *address)
{
	PACKET_INFO p;

	bzero((char *) &p, sizeof(PACKET_INFO));
	p.packet[0] = 0x28;   /*READ(10)*/

	p.packet[2] = (block_num >> 24) & 0xff;
	p.packet[3] = (block_num >> 16) & 0xff;
	p.packet[4] = (block_num >> 8) & 0xff;
	p.packet[5] = block_num & 0xff;
	p.packet[7] = (n >> 8) & 0xff;
	p.packet[8] = n & 0xff;

	p.count = 2048;
	p.drive = device;
	p.address = address;
	atapi_command(&p);
	if (p.status & 1) {
		return -1;
	}
	return 0;
}

