/************************************************************
  Copyright (C) 2001 Nobuhiro Sakawa
  Date: 2001/06/15
  Revision: 1.1 $
*************************************************************/
#ifndef _ide_h
#define	_ide_h

#define	OK					0
#define	NG					1
#define	ON					1
#define	OFF					0

/************************************************
 * PCMCIA Window Offset  MR-SHPC01		*
 ************************************************/
#define	ATTRIBUTE_OFFSET	  (0x00000)		/* Attribute Memory */
#define	IOMEMORY_OFFSET		  (0x100000)		/* I/O */
#define	COMMON_OFFSET		  (0x100000)		/* Common Memory */
#define	UNIT_ADDRESS 		(0xb8400000)

/************************************************
 * Super io SMC37c935a				*
 ************************************************/
#define PA_SUPERIO		(0xb0400000)

/************************************************
 * I/O Base Address And Command Port Address 	*
 ************************************************/
#define	IOBASE_PRIMARY		(0x1f0)		/* I/O Base Address(Primary) 	*/
#define	COMMAN_PORT_PR		(0x3f6)		/* I/O Base Address(Primary) 	*/
#define	IOBASE_SECONDARY	(0x170)		/* I/O Base Address(Secondary) 	*/
#define	COMMAN_PORT_SEC		(0x376)		/* I/O Base Address(Secondary) 	*/
#define	IDE_DISK_MAX		(4)			/* PC Card Max Num */

/************************************************
 *	Register offsets			*
 ************************************************/
#define ERROR_REG		0x1
#define FT_REG			0x1
#define SECTOR_COUNT		0x2
#define SECTOR_NUMBER		0x3
#define LCYLINDER_NUMBER	0x4
#define HCYLINDER_NUMBER	0x5
#define DH_REG			0x6
#define STATUS_REG		0x7
#define WRITE_PRECOMP		0x1
#define COMMAND_REG		0x7
#define DATA_REG		0x0

/************************************************
 *	Commands				*
 ************************************************/
#define ACKNOWLEDGE_MEDIA_CHANGE	0xdb
#define CHECK_POWER_MODE		0x98
#define DOOR_LOCK			0xde
#define DOOR_UNLOCK			0xdf
#define EXECUTE_DRIVE_DIAGNOSTIC	0x90
#define FORMAT_TRACK			0x50
#define IDENTIFY_DRIVE			0xec
#define IDLE				0x97
#define IDLE_IMMEDIATE			0x95
#define INITIALIZE_DRIVE_PARAMETERS	0x91
#define ATAPI_SOFT_RESET		0x08
#define ATAPI_PACKET_COMMAND		0xa0
#define ATAPI_IDENTIFY_DEVICE		0xa1
#define NOP				0x00
#define HD_READ_BUFFER			0xe4
#define READ_DMA_W_RETRY		0xc8
#define READ_DMA_WO_RETRY		0xc9
#define READ_LONG_W_RETRY		0x22
#define READ_LONG_WO_RETRY		0x23
#define READ_MULTIPLE			0xc4
#define READ_SECTOR_W_RETRY		0x20
#define READ_SECTOR_WO_RETRY		0x21
#define READ_VERIFY_SECTOR_W_RETRY	0x40
#define READ_VERIFY_SECTOR_WO_RETRY	0x41
#define RECALIBRATE			0x10
#define SEEK				0x70
#define SET_FEATURES			0xef
#define SET_MULTIPLE_MODE		0xc6
#define SLEEP				0x99
#define	STANDBY				0x96
#define STANDBY_IMMEDIATE		0x94
#define	HD_WRITE_BUFFER			0xe8
#define WRITE_DMA_W_RETRY		0xca
#define WRITE_DMA_WO_RETRY		0xcb
#define WRITE_LONG_W_RETRY		0x32
#define WRITE_LONG_WO_RETRY		0x33
#define WRITE_MULTIPLE			0xc5
#define HD_WRITE_SAME			0xe9
#define WRITE_SECTOR_W_RETRY		0x30
#define WRITE_SECTOR_WO_RETRY		0x31
#define WRITE_VERIFY			0x3c

/************************************************
	Status bits
 ************************************************/
#define BSY_STATUS			0x80
#define RDY_STATUS			0x40
#define WF_STATUS			0x20
#define SC_STATUS			0x10
#define DRQ_STATUS			0x08
#define CRD_STATUS			0x04
#define IDX_STATUS			0x02
#define ERR_STATUS			0x01

/************************************************
	Error register bits
 ************************************************/
#define BB_ERROR			0x80
#define CRC_ERROR			0x40
#define ID_ERROR			0x10

/************************************************
Hard Disk Auxiliary Control Register (HDFDR)
 ************************************************/
#define HD_RST		(0x04)		/* S/W Reset */
#define HD_IDS		(0x02)		/* Interrupt Disable */
#define HD_HS3E		(0x08)		/* fix */
#define HDINFO		((unsigned char *)(1024+32))

struct geometry {
	short         cylinders;
	unsigned char heads;
	unsigned char sectors;
};

struct drive_information {
	unsigned short general_info;
	unsigned short num_cylinders;
	unsigned short reserved1;
	unsigned short num_heads;
	unsigned short filler1[2];			
	unsigned short sectors_per_track;
	unsigned short filler2[3];
	unsigned short serial_number[10];
	unsigned short filler3[3];
	unsigned short firmware_revision[4];
	unsigned short model_number[20];
	unsigned short max_transfered_sectors_per_interrupt;
	unsigned short reserved2;
	unsigned short capabilities;
	unsigned short reserved3;
	unsigned short filler4[2];
	unsigned short valid_info;
	unsigned short now_num_sectors;
	unsigned short now_num_heads;
	unsigned short now_num_sectors_per_track;
	unsigned short now_capacity_sectors[2];
	unsigned short now_multiple_setting;
	unsigned short user_addressable_sectors[2]; /* LBA mode */
	unsigned short filler5;
	unsigned short reserved4;
	unsigned short dma_modes;
	unsigned short min_dma_time;
	unsigned short recomended_dma_time;
	unsigned short min_pio_time_wo_flow_control;
	unsigned short min_pio_time_w_flow_control;
};

struct partition_info {
	unsigned long start;
	unsigned long size;
	unsigned long raw_size;
	int in_use;
};

#define NUM_PARTITIONS	4
struct drive_info {
	struct geometry devinfo;
	long soft_errors;
	struct partition_info part[NUM_PARTITIONS+1];
};

#endif /*_ide_h*/
/**** END OF  IDE_H FILE ****/
