/************************************************************
  ide_load.c ( ide ext2 filesystem load)
  Copyright (C) 2000,2001 Nobuhiro Sakawa
  Date: 2001/06/14
  Revision: 1.0 $
*************************************************************/
# if 0
#define DEBUG 
#endif

#include "ext2_fsb.h"
#include "elfb.h"

#define EXT2_BSIZE	1024

struct ext2_inode *get_inode();
long i_block();
int iperblk;
unsigned long log_block_size;

extern char *load_address;

int block_size ;
int bmul ;
int inode_size;

static unsigned char super_block[4096];
static unsigned char inode_buf[4096];
static unsigned char dir_block[4096];
static unsigned char dir_iblock[4096];
static int pointer_block[2048];
static int pointer_iblock[2048];

static fdump(block)
char *block;
{
	dbcommand(0,block,block + 1024);
}


long ide_load(s, get_block)
char *s;
int (*get_block)();
{
	struct ext2_super_block *sbp;
	struct ext2_inode *dir_ino;
	struct elf32hed   *elfhead;
	long block;
	struct ext2_dir_entry *dir;
	char *temp, *s_save;
	int len;
	long ino;
	long last_block;
	int doff;
	int numdblocks;
	int i,er,ibnum,s2set;
	unsigned long s1last_addr,s2start_addr;
	int disp_cnt;

	block_size = EXT2_BSIZE;
	bmul = 2;
	s2set = 0;

	if ((*get_block)(1L, super_block)) {
	  	sshprintk("get_block error \n");
		fdump(super_block);
		return -1;
	}

	sbp = (struct ext2_super_block *)super_block;
	if(sbp->s_magic !=  EXT2_SUPER_MAGIC) {
		sshprintk("ext2 magic error \n");
		return -1;
	}
	inode_size = EXT2_INODE_SIZE(sbp);
	iperblk = block_size/inode_size;
	log_block_size = sbp->s_log_block_size;
#ifdef DEBUG
sshprintk("super_block iperblk \n");
fdump(super_block);
#endif
	s_save = s;
	ino = 2;
	ibnum = 1;
	while( *s == '/' ) s++;
	len = strlen(s);
	if (!(dir_ino = get_inode(ino, ibnum, get_block))) {
		sshprintk("get_inode error 1\n");
		return -1;
	}
	while((dir_ino->i_mode & 0xf000) != 0x4000 ) { /* dir node ? */
		ibnum++;
		if (!(dir_ino = get_inode(ino, ibnum, get_block))) {
			sshprintk("get_inode error 2\n");
			return -1;
		}
	}
	block = 0;
	last_block = dir_ino->i_blocks;
	numdblocks = dir_ino->i_size / inode_size; 
	for ( ; block <= last_block; ++block) {
		if (read_dir(dir_ino, block, dir_block, get_block)) {
			sshprintk("read_dir error \n");
			return -1;
		}
		if (*(unsigned short *)(dir_block) != 0 ) {
			for (dir = (struct ext2_dir_entry *)(&dir_block[0]);
					  (char *)dir < (char *)&dir_block[512];
					  dir = (struct ext2_dir_entry *)
				      ((char *)dir + dir->rec_len )) {
				if (dir->inode) {
					if (len == dir->name_len && !strncmp(s, dir->name, len)) {
						s += len;
						ino = dir->inode;
						goto found; /*continue;*/
					}
				}
			}
		}
	}
	sshprintk(s_save);
	sshprintk("%s not found 2\n");
	return -1;
found:;
	if (!(dir_ino = get_inode(ino, ibnum, get_block))) {
	    sshprintk("get_inode dir_ino error\n");
		return -1;
		}
#ifdef DEBUG
printk("i_size\n");
fdump(dir_ino);
#endif
	if (dir_ino->i_size) {
		last_block = (dir_ino->i_size - 1) / block_size;
		if(log_block_size) last_block = last_block/(1<<log_block_size) +1;
		disp_cnt = 0;
		for (block = 0; block <= last_block; ++block) {
			/*** oasobi ***/
			if(disp_cnt++ > 40) {	sshprintk(".");	disp_cnt = 0;}
			/**************/
			if (read_block(dir_ino, block, (char *)load_address, get_block)) {
			    sshprintk("read-block error \n");
				return -1;
			}
			if(block == 0) {
				elfhead = (struct elf32hed *)load_address;
				s1last_addr  = elfhead->sh1_entry + elfhead->sh1_size;
				s2start_addr = elfhead->sh2_entry;
			}
			if (block != (EXT2_DIND_BLOCK - 1) ) {
				load_address += block_size<<log_block_size; 
				if(load_address >= (char *)s1last_addr ) {
					if(load_address < (char *)s2start_addr ) {
						if (!s2set) { s2set = 1;
							load_address += 0x1000;
/*nsakawa korega seikai?
							load_address = (char *)s2start_addr;
*/
						}
					}
				}
			}
		}
	} else {
		sshprintk("Empty kernel file!\n");
		return -1;
	}

	return normal_long(dir_ino->i_size);
}

struct ext2_inode *get_inode(n, block_num, get_block)
long n;
long block_num;
int (*get_block)();
{
	long block;
	struct ext2_inode *ip;

	block = --n / iperblk + block_num;
	if ((*get_block)(block, inode_buf)) {
		sshprintk("get_block returned error for block no ");
		hwprintk(block); sshprintk("\n");
		return 0;
	}
	ip = (struct ext2_inode *) (inode_buf + (inode_size * (n % iperblk)));
	return ip;
}


int
read_dir(ino, block, buf, get_block)
struct ext2_inode *ino;
long block;
unsigned char *buf;
int (*get_block)();
{
	int iblock;
	iblock = i_block(ino, block);
	iblock = iblock << log_block_size;

	if ((*get_block)(iblock, buf))
		return -1;
	return 0;
}

int
read_block(ino, block, buf, get_block)
struct ext2_inode *ino;
long block;
unsigned char *buf;
int (*get_block)();
{
	int i,iblock,pib, pob, pibsize;

	if (block < (EXT2_DIND_BLOCK - 1)) {
		iblock = i_block(ino,block);
		if(!log_block_size) {
			if ((*get_block)(iblock, buf))
				return -1;
		}else{
			for( i=0; i < (1<<log_block_size); i++) {
				if ((*get_block)((iblock<<log_block_size)+i,
								   buf+(EXT2_BSIZE*i)))
					return -1;
			}
		}
		return 0;
	} 

	if (block == (EXT2_DIND_BLOCK - 1) ) {
		iblock = i_block(ino, block);
		if(!log_block_size) {
			if ((*get_block)(iblock, pointer_block))
				return -1;
		}else{
			for( i=0; i < log_block_size; i++) {
				if ((*get_block)((iblock<<log_block_size)+i,
						pointer_block+(EXT2_BSIZE/sizeof(int)*i)))
					return -1;
			}
		}
		return 0;
	}
	if(log_block_size){
		pibsize = (block_size*log_block_size) / sizeof(int);
	}else{
		pibsize = block_size / sizeof(int);
	}
	if (block == ( EXT2_DIND_BLOCK + pibsize - 1 )) {
		if ((*get_block)(i_block(ino, EXT2_DIND_BLOCK)<<log_block_size, pointer_iblock))
			return -1;
	}
	
	pib = (block - EXT2_DIND_BLOCK ) / pibsize;
	pob = (block - EXT2_DIND_BLOCK ) % pibsize;
	if ((pob == 0) && (pib != 0)) {
		if ((*get_block)(pointer_iblock[pib-1]<<log_block_size, pointer_block))
			return -1;
	}
	iblock = pointer_block[pob];
	if(!log_block_size) {
		if ((*get_block)(iblock, buf))
			return -1;
	}else{
		for( i=0; i < (1<<log_block_size); i++) {
			if ((*get_block)((iblock<<log_block_size)+i,
							   buf+(EXT2_BSIZE*i)))
				return -1;
		}
	}

	return 0;
}

long i_block(d, x)
struct ext2_inode *d;
long x;
{
    return d->i_block[x];
}
