/************************************************************
  iderbk.c ( ide disk deiver )
  Copyright (C) 2001 Nobuhiro Sakawa
  Date: 2001/01/01
  Revision: 1.0 $
*************************************************************/
/*
idebrk.c 			IDE Disk READ
*/

#define  BLOCK_SIZE 512

#include "ide.h"
#include "bios.h"
#include "config.h"

#define SYSERR		-1
#ifndef FALSE
#define FALSE		0
#endif
#ifndef TRUE
#define TRUE		1
#endif
#ifndef NULL
#define NULL		0
#endif

#ifdef 	READ
#undef	READ
#endif
#define READ 0

#ifdef	WRITE
#undef	WRITE
#endif
#define WRITE 1

/************************* debug mode *************************/
#if 0
#define	DBG 
#endif

extern int boot_dev;
extern int boot_partition;
extern int minor_device;
extern int disk_error;
extern int bmul;


static int error;
static struct drive_info driveinfo;
static unsigned char datablock[512];
static unsigned int iobase;   /* we boot off the primary disk only */
static unsigned int command;  /* command port */
struct geometry	ide_info;

static unsigned char *IOBASE ;

unsigned int devtype;
#define IDE_DEV 0
#define CD_DEV  1
#define LBA_DEV 2

static	void wait()
{
int	i;
	for(i=0;i<2;i++) {}
}

static	void wait500ms()
{
unsigned long	about500ms = ((50*1000)*1000) / 100;
unsigned int	i ;
	for ( i = 0 ; i<about500ms ; i++) {}
}

static inline volatile unsigned short *
portshift(port)
unsigned int port;
{
	return (volatile unsigned short *)(PA_SUPERIO + ( port << 1));
}

void	outb(port,val) 
unsigned short	port;
unsigned char	val; 
{
	wait();
	wait();
	if (boot_dev ==  1 ) {
		*(volatile unsigned char *)(IOBASE + port) = val ;
	} else {
		*portshift(port) = val;
	}
}
unsigned char inb(port) 
unsigned short	port;
{
unsigned char	rc ;
	wait();
	wait();
	if (boot_dev ==  1 ) {
		rc = *(volatile unsigned char *)(IOBASE + port);
	} else {
		rc = (unsigned char)((*portshift(port))&0xff);
	}
	return ( rc ) ;
}

void insw(port, buffer, count)
short 	port;
short 	*buffer;
int 	count;
{
int	i;
unsigned short	*rp ;
	if (boot_dev ==  1 ) {
		rp = (unsigned short*)(IOBASE + port) ;
	} else {
		rp = (unsigned short*)(portshift(port)) ;
	}
	for(i=0;i<count;i++)
	{
		wait();
		*buffer++ = *rp;
	}
}
void outsw(port, buffer, count)
short 	port;
short 	*buffer;
int 	count;
{
int	i;
unsigned short	*rp ;
	if (boot_dev ==  1 ) {
		rp = (unsigned short*)(IOBASE + port) ;
	} else {
		rp = (unsigned short*)(portshift(port)) ;
	}
	for(i=0;i<count;i++)
	{
		wait();
		*rp = *buffer++;;
	}
}

static void idewait()
{
	while (inb(iobase + STATUS_REG) & BSY_STATUS)
		;
}

static	identify_device(drive,driveinfo)
struct drive_info *driveinfo;
{
	struct drive_information *infoptr; 

	outb(iobase + DH_REG, 0xa0 | drive<<4); 
	outb(iobase + COMMAND_REG, IDENTIFY_DRIVE);  /* get drive information */
	idewait();
	insw(iobase + DATA_REG, datablock, 256);
	infoptr = (struct drive_information *) datablock;
	driveinfo->devinfo.cylinders = infoptr->num_cylinders; 
	driveinfo->devinfo.heads     = infoptr->num_heads; 
	driveinfo->devinfo.sectors   = infoptr->sectors_per_track; 

    if(infoptr->capabilities & 0x0200) devtype = LBA_DEV;
    else devtype = IDE_DEV;
}

ide_open() 
{
	int drive, partition,i;
	unsigned char test_byte;
	unsigned char	block[BLOCK_SIZE];
	unsigned char	*unit_adress;
	unsigned long   wtmp;

	drive              = minor_device;
	partition          = boot_partition;
	ide_info.cylinders = 0; 
	ide_info.heads     = 0; 
	ide_info.sectors   = 0; 

	driveinfo.devinfo = * (struct geometry *)&ide_info ;
	IOBASE  = (unsigned char*) IO_WIN;
	if (boot_dev == 1 ) {  
		iobase 	= IO_BASE_OFFSET ;	/* I/O port */
	} else {
		iobase 	= IO_BASE_OFFSET_IDE ;	/* Super I/O port */
	}
	command = 0xe ; 		/* command port */
	
	outb(iobase + DH_REG, 0xa0 | drive<<4);
	
	if ((inb(iobase + HCYLINDER_NUMBER) == 0xeb) &&
		(inb(iobase + LCYLINDER_NUMBER) == 0x14) ) {
		sshprintk("Boot Device ATAPI !! notSoport \n");
		devtype = CD_DEV; 
	}
	else {
	/* Test to see if ide card is present */ 
		wtmp = 0;
		outb(iobase + LCYLINDER_NUMBER, 0x5a);
		while( (test_byte=inb(iobase + LCYLINDER_NUMBER)) != 0x5a) {
			if(wtmp > 10) {
				sshprintk("IDE adaptor not found\n");
				return -1;
			}
			outb(iobase + LCYLINDER_NUMBER, 0x5a);
			wait500ms();
			wtmp++;
		}
		outb(command, HD_HS3E); /* Interrupts on for ide disk */
		if (identify_device(drive,&driveinfo) == SYSERR) {
			sshprintk("identify device not supported on drive %d\n");
			hwprintk(drive); sshprintk("\n");
		}
		error = 0;
		do_restore(drive);		/* test for disk and set seek rate */
		if (error) {
			sshprintk("Cannot restore hard driver\n");
			return -1;
		}
	}
	
	error = 0;
	if (devtype == CD_DEV ) {
	/*
		cd_set_params(drive, driveinfo.devinfo.heads,
			driveinfo.devinfo.sectors);
	*/
	} else {
		ide_set_params(drive, driveinfo.devinfo.heads,
			driveinfo.devinfo.sectors);
	}
	if (error) sshprintk("Set Device parameter Error \n");

	error = 0;
	if (partition == 0) {			/* whole disk */
		driveinfo.part[0].raw_size = driveinfo.devinfo.cylinders * driveinfo.devinfo.heads * driveinfo.devinfo.sectors;
		driveinfo.part[0].start = 0;
		driveinfo.part[0].size = driveinfo.part[0].raw_size - driveinfo.devinfo.heads * driveinfo.devinfo.sectors;
	}
	else {		/* parttition != 0 */
		error = 0;
		ide_r_block(1, 0L, drive, block);
		if (error) {
			sshprintk("Cannot read block 0\n");
			return -1;
		}
		if (block[0x1fe] != 0x55 || block[0x1ff] != 0xaa) {
			sshprintk("Partition magic error 55aa=");
			hbprintk(block[0x1fe]); hbprintk(block[0x1ff]);
			sshprintk("\n");
			return -1;
		}
		wtmp=partition*16 + (0x1c6 - 16);
		driveinfo.part[partition].start = 
		   block[wtmp] | block[wtmp+1]<<8 | block[wtmp+2]<<16 | block[wtmp+3]<<24;
		wtmp = wtmp + 4;
		driveinfo.part[partition].raw_size = driveinfo.part[partition].size =
		   block[wtmp] | block[wtmp+1]<<8 | block[wtmp+2]<<16 | block[wtmp+3]<<24;
	}
#ifdef DBG
sshprintk("ide_open()\n");
#endif
	return OK;
}

ide_close()
{
}

do_restore(drive)
{
	outb(iobase + DH_REG, 0xa0 | drive<<4);
	outb(iobase + COMMAND_REG, RECALIBRATE);
	idewait();
	error = inb(iobase + STATUS_REG) & ERR_STATUS;
}

ide_set_params(drive, num_heads, sec_per_track)
int drive;
int num_heads;

{
	outb(iobase + DH_REG, 0xa0 | drive<<4 | (num_heads-1));
	outb(iobase + SECTOR_COUNT, sec_per_track);
	outb(iobase + COMMAND_REG, INITIALIZE_DRIVE_PARAMETERS);
	idewait();
	error = inb(iobase + STATUS_REG) & ERR_STATUS;
}
#if 0
cd_set_params(drive, num_heads, sec_per_track)
int drive;
int num_heads;

{
	outb(iobase + DH_REG, 0xa0 | drive<<4 );
	outb(iobase + COMMAND_REG, ATAPI_IDENTIFY_DEVICE);
	idewait();
	error = inb(iobase + STATUS_REG) & ERR_STATUS;
}
#endif
int ide_read_block(block, address)
{
#ifdef DBG
sshprintk("ide_r_block=");
hwprintk(block); sshprintk("\n");
#endif
	return ide_r_block(bmul, block*bmul + driveinfo.part[boot_partition].start, 
		minor_device, address);
}

int ide_r_block(n, block_num, device,  address)
int device;
long block_num;
char *address;
{
	register int i;
	int sector, cylinder, head;
	int drive ;
	char status, error;
	int num_heads, num_sectors;
	long x;

	drive = device & 1;
	if(devtype == LBA_DEV) {
		outb(iobase + SECTOR_COUNT, n);
		outb(iobase + SECTOR_NUMBER, block_num & 0xff);
		outb(iobase + LCYLINDER_NUMBER, (block_num >> 8) & 0xff);
		outb(iobase + HCYLINDER_NUMBER, (block_num >> 16) & 0xff);
		outb(iobase + DH_REG,
				0x40 | (drive << 4) | ((block_num >> 24) & 0x0f));
	} else {
		num_heads = driveinfo.devinfo.heads;
		num_sectors = driveinfo.devinfo.sectors;
		sector = block_num % num_sectors + 1;
		x = block_num / num_sectors;
		head = x % num_heads + 0xa0 + (drive<<4);
		cylinder = x / num_heads;
		outb(iobase + SECTOR_COUNT, n);
		outb(iobase + SECTOR_NUMBER, sector);
		outb(iobase + LCYLINDER_NUMBER, cylinder & 0xff);
		outb(iobase + HCYLINDER_NUMBER, (cylinder >> 8) & 0xff);
		outb(iobase + DH_REG, head);
	}
	
	disk_error = 0;

	outb(iobase + COMMAND_REG, READ_SECTOR_W_RETRY);
	for (i = 0; i < n; i++) {
		idewait();
		insw(iobase + DATA_REG, address, 256);
		address+=BLOCK_SIZE;
	}
	status = inb(iobase + STATUS_REG);
	error = inb(iobase + ERROR_REG);
	if (status & ERR_STATUS) {
		sshprintk("HARD READ ERROR block ");
		hwprintk(block_num); sshprintk("\n");
		disk_error = 1;
	}
	else if ((status & CRD_STATUS) || (error & (CRC_ERROR|ID_ERROR))) {
		sshprintk("SOFT READ ERROR block");
		hwprintk(block_num); sshprintk("\n");
	}

	if (disk_error)
		return -1;
	return 0;
}
