/************************************************************
  sh4bios.c (sh BIOS main )
  Copyright (C) 2001 Nobuhiro Sakawa
  Date: 2001/06/15
  Revision: 1.1 $
*************************************************************/
#define	MAIN
#define	IDE
#include "config.h"
#include "sh4bios.h"
#include "ide.h"
#include "sh.h"

extern int keysw;
extern int screeny;
extern long normal_long();
extern sshprintk();

#define MAX_STRING		80
char	buffer[MAX_STRING];
int 	boot_drv;
int 	boot_dev;
int 	minor_device;
int 	disk_error;
int 	boot_partition;
char	*load_address;
char	*start_address;

static (*close_routine)();
static (*open_routine)();
static (*read_routine)();

char *skip_space(char *p)
{
	while (*p == ' ' || *p == '\t')  p++;
	return p;	
}

char *next_word(char *p, char *buff)
{
	int i;

	p = skip_space(p);
	i = 0;
	while (*p && *p != ' ' && *p != '\t') {
		if (++i < MAX_STRING)	*buff++ = *p++;
	}
	*buff = 0;
	return p;
}

void do_boot(char *p)
{
	char buffer[MAX_STRING];

	p++;
	p = next_word(p, buffer);
	if (buffer[0] == '/') {
		if (*p)
			sshprintk("Syntax Error\n");
		else {
			sshprintk("Booting from file ");
			sshprintk(buffer); sshprintk("\n");
			boot(buffer);
			return;
		}
	}
	if (*p)
		sshprintk("Syntax Error\n");
	else
		boot("/vmlinux");
}

void hd_do_boot(p)
char *p;
{
	boot_drv = 0;  /* devtype==?  ide=0 CD=1 */
	boot_dev = 0;  /* IDE  */
	do_boot(p);
}

void cd_do_boot(p)
char *p;
{
	boot_drv = 1;  /* devtype==?  ide=0 CD=1 */
	boot_dev = 0;  /* IDE */
	do_boot(p);
}
void cf_do_boot(p)
char *p;
{
	boot_drv = 0;  /* devtype==?  ide=0 CD=1 */
	boot_dev = 1;  /* se_sh4 pcmcia */
	do_boot(p);
}

int boot(pathname)
char *pathname;
{
	int dev;
	int ide_open(), ide_read_block(), ide_close();
	int cd_open(), cd_read_block(), cd_close();

	if (!boot_drv) { /* ide HDD /dev/hda1 */
		open_routine  = ide_open;	
		read_routine  = ide_read_block;
		close_routine = ide_close;
		minor_device  = 0;
		boot_partition = 1;
	}
	else {           /* atapi CDROM /dev/hdb */
		open_routine  = cd_open;	
		read_routine  = cd_read_block;
		close_routine = cd_close;
		minor_device  = 1;
		boot_partition = 0;
	}

	if ((*open_routine)() != OK) {
		sshprintk("Cannot open device \n"); 
		return ;
	}

	load_address     = (char *)LOAD_ADDRESS;  
	start_address    = (char *)START_ADDRESS; 

	sshprintk("Loading...");
	if (!boot_drv) {
		if (ide_load(pathname, read_routine) == -1) {
			sshprintk("Loading Error!   file "); 
			sshprintk(pathname); sshprintk("  /dev/hda1 \n"); 
			(*close_routine)();
			return;
		}
	}
	else {
		if (cd_load(pathname, read_routine) == -1) {
			sshprintk("Loading Error!   file "); 
			sshprintk(pathname); sshprintk(" /dev/hdb \n"); 
			(*close_routine)();
			return;
		}
	}
	if (disk_error) {
		sshprintk("Disk Error \n ");
		return;
	}
	sshprintk("loaded\n");

	/********************************************************/
	/*	linux start infmation 				*/
	/********************************************************/
	MOUNT_ROOT_RDONLY = 0X00000000; 
	RAMDISK_FLAGS     = 0X00000000; 
	if (boot_drv == 0){
		ORIG_ROOT_DEV     = 0X00000301; /*hda1*/
	} else {
		ORIG_ROOT_DEV     = 0X00000340; /*hdb*/
	}
	INITRD_START      = 0X00000000; 
	INITRD_SIZE       = 0X00000000; 
	/*********************************************************/

	(*(int(*)())start_address)();
	return 0;
}

main()
{
	keysw = 0;
	boot_dev = 0;
	sio_init();			/* SH  SIO initial */
	sshprintk("\n\n\n\n");
	sshprintk(VERSION);
	sshprintk("\n     Copyright (C) 2001,2002  Nobuhiro sakawa \n");

#if defined(SE01SH4)
	PCIC_card_init(1 /* ATR*/);
	smsc_init();
#endif
#if defined(KZP01)
	pci_int();		/* SH4 PCI   */
	pci_br_init();		/* PCI Bridge*/
	pci_sio_init(); 	/* PCI SIO   */
	ide_init();		/* IDE       */
	VGA_init();		/* VGA       */
	KEY_init();		/* KEY       */

	/* VGA display */
	dsp_init();
    	crt_str(VERSION,2,5,0x07,1);
    	crt_str(" Copyright (C) 2001  Nobuhiro sakawa",2,7,0x07,1);
	#if 0  /* == 1 console  sci --> vga */
    		keysw   = 1;
    		screeny = 8;
	#endif
#endif

	sshprintk("Command? ");
	for(;;) {
		cmd();
	}
}

